<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit;
}

require_once SD_TRUST_PATH . '/admin/class/installer/SdInstallUtils.class.php';

/**
 * Sd_Installer
**/
class Sd_Installer
{
    /**
     * @var  Legacy_ModuleInstallLog
     * 
     * @public
    **/
    var $mLog = null;

    /**
     * @var  bool
     * 
     * @private
    **/
    var $_mForceMode = false;

    /**
     * @var  XoopsModule
     * 
     * @private
    **/
    var $_mXoopsModule = null;

    /**
     * Sd_Installer
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @public
    **/
    function Sd_Installer()
    {
        $this->mLog =& new Legacy_ModuleInstallLog();
    }

    /**
     * setCurrentXoopsModule
     * 
     * @param   XoopsModule  &$xoopsModule
     * 
     * @return  void
     * 
     * @public
    **/
    function setCurrentXoopsModule(&$xoopsModule)
    {
        $this->_mXoopsModule =& $xoopsModule;
    }

    /**
     * setForceMode
     * 
     * @param   bool  $isForceMode
     * 
     * @return  void
     * 
     * @public
    **/
    function setForceMode($isForceMode)
    {
        $this->_mForceMode = $isForceMode;
    }

    /**
     * _installTables
     * 
     * @param   void
     * 
     * @return  bool
     * 
     * @private
    **/
    function _installTables()
    {
        return Sd_InstallUtils::installSQLAutomatically(
            $this->_mXoopsModule,
            $this->mLog
        );
    }

    /**
     * _installModule
     * 
     * @param   void
     * 
     * @return  bool
     * 
     * @private
    **/
    function _installModule()
    {
        $moduleHandler =& Sd_Utils::getXoopsHandler('module');
        if(!$moduleHandler->insert($this->_mXoopsModule))
        {
            $this->mLog->addError(_MI_SD_INSTALL_ERROR_MODULE_INSTALLED);
            return false;
        }
    
        $gpermHandler =& Sd_Utils::getXoopsHandler('groupperm');
    
        if($this->_mXoopsModule->getInfo('hasAdmin'))
        {
            $adminPerm =& $this->_createPermission(XOOPS_GROUP_ADMIN);
            $adminPerm->setVar('gperm_name','module_admin');
            if(!$gpermHandler->insert($adminPerm))
            {
                $this->mLog->addError(_MI_SD_INSTALL_ERROR_PERM_ADMIN_SET);
            }
        }
    
        if($this->_mXoopsModule->getInfo('hasMain'))
        {
            if($this->_mXoopsModule->getInfo('read_any'))
            {
                    $memberHandler =& Sd_Utils::getXoopsHandler('member');
                    $groupObjects =& $memberHandler->getGroups();
                    foreach($groupObjects as $group)
                    {
                        $readPerm =& $this->_createPermission($group->getVar('groupid'));
                        $readPerm->setVar('gperm_name','module_read');
                        if(!$gpermHandler->insert($readPerm))
                        {
                            $this->mLog->addError(_MI_SD_INSTALL_ERROR_PERM_READ_SET);
                        }
                    }
            }
            else
            {
                    $root =& XCube_Root::getSingleton();
                    $groups = $root->mContext->mXoopsUser->getGroups();
                    foreach($groups as $group)
                    {
                        $readPerm =& $this->_createPermission($group);
                        $readPerm->setVar('gperm_name','module_read');
                        if(!$gpermHandler->insert($readPerm))
                        {
                            $this->mLog->addError(_MI_SD_INSTALL_ERROR_PERM_READ_SET);
                        }
                    }
            }
        }
    
        return true;
    }

    /**
     * &_createPermission
     * 
     * @param   int  $group
     * 
     * @return  XoopsGroupPerm
     * 
     * @private
    **/
    function &_createPermission($group)
    {
        $gpermHandler =& Sd_Utils::getXoopsHandler('groupperm');
        $perm =& $gpermHandler->create();
        $perm->setVar('gperm_groupid',$group);
        $perm->setVar('gperm_itemid',$this->_mXoopsModule->getVar('mid'));
        $perm->setVar('gperm_modid',1);
    
        return $perm;
    }

    /**
     * _installTemplates
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function _installTemplates()
    {
        Sd_InstallUtils::installAllOfModuleTemplates(
            $this->_mXoopsModule,
            $this->mLog
        );
    }

    /**
     * _installBlocks
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function _installBlocks()
    {
        Sd_InstallUtils::installAllOfBlocks(
            $this->_mXoopsModule,
            $this->mLog
        );
    }

    /**
     * _installPreferences
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function _installPreferences()
    {
        Sd_InstallUtils::installAllOfConfigs(
            $this->_mXoopsModule,
            $this->mLog
        );
    }

    /**
     * _processReport
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function _processReport()
    {
        if(!$this->mLog->hasError())
        {
            $this->mLog->add(
                XCube_Utils::formatString(
                    _MI_SD_INSTALL_MSG_MODULE_INSTALLED,
                    $this->_mXoopsModule->getInfo('name')
                )
            );
        }
        else if(is_object($this->_mXoopsModule))
        {
            $this->mLog->addError(
                XCube_Utils::formatString(
                    _MI_SD_INSTALL_ERROR_MODULE_INSTALLED,
                    $this->_mXoopsModule->getInfo('name')
                )
            );
        }
        else
        {
            $this->mLog->addError(
                XCube_Utils::formatString(
                    _MI_SD_INSTALL_ERROR_MODULE_INSTALLED,
                    'something'
                )
            );
        }
    }

    /**
     * executeInstall
     * 
     * @param   void
     * 
     * @return  bool
     * 
     * @public
    **/
    function executeInstall()
    {
        $this->_installTables();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_installModule();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_installTemplates();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_installBlocks();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_installPreferences();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_installDefaultData();
        if(!$this->_mForceMode && $this->mLog->hasError())
        {
            $this->_processReport();
            return false;
        }
    
        $this->_processReport();
        return true;
    }
    
    /**
     * _installDefaultData
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @private
    **/
    function _installDefaultData()
    {
        require_once SD_TRUST_PATH . '/class/AssetManager.class.php';
        $asset =& Sd_AssetManager::getInstance($this->_mXoopsModule->get('dirname'));
        foreach(array('drop','drop_seed','map_spot') as $type)
        {
            $func = '_insert' . ucfirst($type) . 'Data';
            if($this->$func($asset->getObject('handler',$type)))
            {
                $this->mLog->addReport($type . ' data inserted.');
            }
            else
            {
                $this->mLog->addError('Cannot insert ' . $type . ' data.');
            }
        }
    }
    
    /**
     * _insertDropData
     * 
     * @param   Sd_DropHandler  &$handler
     * 
     * @return  bool
     * 
     * @private
    **/
    function _insertDropData(&$handler)
    {
        foreach(array(_MI_SD_DEFAULT_EMPTY,_MI_SD_DEFAULT_CLOSED) as $value)
        {
            $obj =& $handler->create();
            $obj->setVars(array('drop_name' => $value,'seed' => 1,'element' => -2));
            if(!$handler->insert($obj))
            {
                return false;
            }
            unset($obj);
        }
        return true;
    }
    
    /**
     * _insertDrop_seedData
     * 
     * @param   Sd_Drop_seedHandler &$handler
     * 
     * @return  bool
     * 
     * @private
    **/
    function _insertDrop_seedData(&$handler)
    {
        $obj =& $handler->create();
        $obj->set('drop_seed_name',_MI_SD_DEFAULT_UNKNOWN);
        return $handler->insert($obj);
    }
    
    /**
     * _insertMap_spotData
     * 
     * @param   Sd_Map_spotHandler  &$handler
     * 
     * @return  bool
     * 
     * @private
    **/
    function _insertMap_spotData(&$handler)
    {
        $obj =& $handler->create();
        $obj->set('spot_name',_MI_SD_DEFAULT_UNKNOWN);
        return $handler->insert($obj);
    }
}

?>
