<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit;
}

/**
 * Sd_MapObject
**/
class Sd_MapObject extends XoopsSimpleObject
{
    /**
     * Sd_MapObject
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @public
    **/
    function Sd_MapObject()
    {
        $this->initVar('x', XOBJ_DTYPE_INT, '', false);
        $this->initVar('y', XOBJ_DTYPE_INT, '', false);
        $this->initVar('spot_id', XOBJ_DTYPE_INT, 1, false);
        $this->initVar('update_time', XOBJ_DTYPE_INT, '', false);
    }
}

/**
 * Sd_MapSpotCollection
**/
class Sd_MapSpotCollection implements Iterator
{
    /**
     * @brief   Sd_Map_spotObject{}
    **/
    protected $_mSpot = array();
    
    /**
     * @brief   Enum
    **/
    protected $_mUnknownId = null;
    
    /**
     * @brief   Enum{'x'}{'y'}
    **/
    protected $_mMap = array();
    
    /**
     * @brief   int{}
    **/
    protected $_mMapSize = array();
    
    /**
     * @brief   int{}
    **/
    protected $_mCurrent = array('x' => 0,'y' => 0);
    
    /**
     * @brief   bool
    **/
    protected $_mIsLast = false;
    
    /**
     * __construct
     * 
     * @param   string  $dirname
     * @param   int     $minX
     * @param   int     $minY
     * @param   int     $maxX
     * @param   int     $maxY
     * 
     * @return  void
    **/
    public function __construct(/*** string ***/ $dirname,/*** int ***/ $minX,/*** int ***/ $minY,/*** int ***/ $maxX,/*** int ***/ $maxY)
    {
        $this->_mMapSize = compact('minX','minY','maxX','maxY');
        $this->rewind();
        
        $asset =& Sd_AssetManager::getInstance($dirname);
        $spotHandler =& $asset->getObject('handler','map_spot');
        $this->_mSpot = $spotHandler->getObjects(null,null,null,true);
    }
    
    /**
     * prepare
     * 
     * @param   Enum            $unknownId
     * @param   Enum{'x'}{'y'}  $map
     * 
     * @return  void
    **/
    public function prepare(/*** Enum ***/ $unknownId,array $map)
    {
        $this->_mUnknownId = $unknownId;
        $this->_mMap = $map;
    }
    
    /**
     * get
     * 
     * @param   int     $x
     * @param   int     $y
     * 
     * @return  Sd_Map_spotObject
    **/
    public function get(/*** int ***/ $x,/*** int ***/ $y)
    {
        $id = isset($this->_mMap[$x][$y]) ? $this->_mMap[$x][$y] : $this->_mUnknownId;
        return isset($this->_mSpot[$id]) ? $this->_mSpot[$id] : $this->_mSpot[$this->_mUnknownId];
    }
    
    /**
     * current
     * 
     * @param   void
     * 
     * @return  Sd_Map_spotObject
    **/
    public function current()
    {
        return $this->get($this->_mCurrent['x'],$this->_mCurrent['y']);
    }
    
    /**
     * key
     * 
     * @param   void
     * 
     * @return  int{}
    **/
    public function key()
    {
        return $this->_mCurrent;
    }
    
    /**
     * next
     * 
     * @param   void
     * 
     * @return  void
    **/
    public function next()
    {
        $this->_mCurrent['x']++;
        if($this->_mCurrent['x'] > $this->_mMapSize['maxX'])
        {
            $this->_mCurrent['x'] = $this->_mMapSize['minX'];
            $this->_mCurrent['y']++;
            if($this->_mCurrent['y'] > $this->_mMapSize['maxY'])
            {
                $this->_mIsLast = true;
                $this->_mCurrent['y'] = $this->_mMapSize['maxY'];
            }
        }
    }
    
    /**
     * rewind
     * 
     * @param   void
     * 
     * @return  void
    **/
    public function rewind()
    {
        $this->_mCurrent = array(
            'x' => $this->_mMapSize['minX'],
            'y' => $this->_mMapSize['minY']
        );
        $this->_mIsLast = false;
    }
    
    /**
     * seek
     * 
     * @param   int{}   $key
     * 
     * @return  void
    **/
    public function seek(array $key)
    {
        if(!isset($key['x']) || $key['x'] < $this->_mMapSize['minX'] || $key['x'] > $this->_mMapSize['maxX'] || !isset($key['y']) || $key['y'] < $this->_mMapSize['minY'] ||  $key['y'] > $this->_mMapSize['maxY'])
        {
            return;
        }
        
        $this->_mCurrent = array(
            'x' => $key['x'],
            'y' => $key['y']
        );
        $this->_mIsLast = ($key['x'] == $this->_mMapSize['maxX'] && $key['y'] == $this->_mMapSize['maxY']);
    }
    
    /**
     * valid
     * 
     * @param   void
     * 
     * @return  bool
    **/
    public function valid()
    {
        return !$this->_mIsLast;
    }
    
    /**
     * isMaxX
     * 
     * @param   void
     * 
     * @return  bool
    **/
    public function isMaxX()
    {
        return $this->_mCurrent['x'] == $this->_mMapSize['maxX'];
    }
    
    /**
     * isMaxY
     * 
     * @param   void
     * 
     * @return  bool
    **/
    public function isMaxY()
    {
        return $this->_mCurrent['y'] == $this->_mMapSize['maxY'];
    }
}


/**
 * Sd_MapHandler
**/
class Sd_MapHandler extends XoopsObjectGenericHandler
{
    /**
     * @var  string
     * 
     * @public
    **/
    var $mTable = '{dirname}_map';

    /**
     * @var  string
     * 
     * @public
    **/
    var $mPrimary = 'x';

    /**
     * @var  string
     * 
     * @public
    **/
    var $mClass = 'Sd_MapObject';

    /**
     * @brief   string
    **/
    protected $_mDirname = null;

    /**
     * Sd_MapHandler
     * 
     * @param   XoopsDatabase  &$db
     * @param   string  $dirname
     * 
     * @return  void
     * 
     * @public
    **/
    function Sd_MapHandler(&$db,$dirname)
    {
        $this->mTable = str_replace('{dirname}',$dirname,$this->mTable);
        parent::XoopsObjectGenericHandler($db);
        
        $this->_mDirname = $dirname;
    }
    
    /**
     * get
     * 
     * @param   int $x
     * @param   int $y
     * 
     * @return  Sd_MapObject
    **/
    public function get(/*** int ***/ $x,/*** int ***/ $y)
    {
        $cri = new CriteriaCompo(new Criteria('x',$x));
        $cri->add(new Criteria('y',$y));
        $ret = $this->getObjects($cri);
        
        if(is_array($ret) && count($ret) == 1)
        {
            $ret = $ret[0];
        }
        else
        {
            $ret = $this->create();
            $ret->setVars(array('x' => $x,'y' => $y));
        }
        
        return $ret;
    }
    
    /**
     * _update
     * 
     * @param   Sd_MapObject    &$obj
     * 
     * @return  string
     * 
     * @access  protected
    **/
    public function _update(Sd_MapObject &$obj)
    {
        return sprintf(
            'update `%s` set `spot_id` = %d where `x` = %d and `y` = %d',
            $this->mTable,$obj->get('spot_id'),$obj->get('x'),$obj->get('y')
        );
    }
    
    /**
     * getMapSpotCollection
     * 
     * @param   int     $updateTime
     * 
     * @return  Sd_MapSpotCollection
    **/
    public function getMapSpotCollection(/*** int ***/ $updateTime)
    {
        $cri = new Criteria('update_time',$updateTime,'<=');
        $sql = sprintf('select min(`x`) as min_x,min(`y`) as min_y,max(`x`) as max_x,max(`y`) as max_y from `%s` where `update_time` <= %d',$this->mTable,$updateTime);
        
        if(!$res = $this->db->query($sql))
        {
            return null;
        }
        
        $size = $this->db->fetchArray($res);
        $dummy = $this->create();
        $map = array();
        foreach($this->getObjects($cri) as $obj)
        {
            $map[$obj->get('x')][$obj->get('y')] = $obj->get('spot_id');
        }
        
        $spotCollection = new Sd_MapSpotCollection($this->_mDirname,$size['min_x'],$size['min_y'],$size['max_x'],$size['max_y']);
        $spotCollection->prepare($dummy->get('spot_id'),$map);
        
        return $spotCollection;
    }
}

?>
